Re: [PATCH v2 07/10] strmap: enable faster clearing and reusing of strmaps
- From
Jeff King <peff@peff.net>
- Date
- Oct 30, 2020, 14:27 UTC
- Message-ID
- <20201030142707.GG3277724@coredump.intra.peff.net>
- In-Reply-To
- <2ebce0c5d82b87fa9c9ef5dcefc0ac2701654f3b.1602549650.git.gitgitgadget@gmail.com>
On Tue, Oct 13, 2020 at 12:40:47AM +0000, Elijah Newren via GitGitGadget wrote:
Show 16 quoted lines
> From: Elijah Newren <newren@gmail.com> > > When strmaps are used heavily, such as is done by my new merge-ort > algorithm, and strmaps need to be cleared but then re-used (because of > e.g. picking multiple commits to cherry-pick, or due to a recursive > merge having several different merges while recursing), free-ing and > reallocating map->table repeatedly can add up in time, especially since > it will likely be reallocated to a much smaller size but the previous > merge provides a good guide to the right size to use for the next merge. > > Introduce strmap_partial_clear() to take advantage of this type of > situation; it will act similar to strmap_clear() except that > map->table's entries are zeroed instead of map->table being free'd. > Making use of this function reduced the cost of reset_maps() by about > 20% in mert-ort, and dropped the overall runtime of my rebase testcase > by just under 2%.
Oh, these were the real numbers I was looking for earlier. :)
Of course it's a little confusing because reset_maps() doesn't exist yet in the code base this is being applied on, but I can live with that.
Show 5 quoted lines
> +/* > + * Similar to strmap_clear() but leaves map->map->table allocated and > + * pre-sized so that subsequent uses won't need as many rehashings. > + */ > +void strmap_partial_clear(struct strmap *map, int free_values);
Oh good, you anticipated my free_values suggestion from earlier. But...
Show 5 quoted lines
> +void strmap_partial_clear(struct strmap *map, int free_util)
> +{
> + strmap_free_entries_(map, free_util);
> + hashmap_partial_clear(&map->map);
> +}...the implementation didn't catch up.
Other than that the patch looks obviously correct.
-Peff